/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
#include "user\temperature.h"
#include "delays.h"
/** V A R I A B L E S ********************************************************/
#pragma udata
byte old_sw2,old_sw3;
byte counter;
byte trf_state;
byte temp_mode;

DATA_PACKET dataPacket;

byte pTemp;                     // Pointer to current logging position, will
                                // loop to zero once the max index is reached
byte valid_temp;                // Keeps count of the valid data points
word temp_data[30];             // 30 points of data

// Timer0 - 1 second interval setup.
// Fosc/4 = 12MHz
// Use /256 prescalar, this brings counter freq down to 46,875 Hz
// Timer0 should = 65536 - 46875 = 18661 or 0x48E5
#define TIMER0L_VAL         0xE5
#define TIMER0H_VAL         0x48

/** P R I V A T E  P R O T O T Y P E S ***************************************/

void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);
void ResetTempLog(void);
void ReadPOT(void);
void ServiceRequests(void);

// For board testing purpose only
void PICDEMFSUSBDemoBoardTest(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
    //mInitAllLEDs();
    mInitAllSwitches();
    old_sw2 = sw2;
    old_sw3 = sw3;
    
    //set up charge pump system.
    
    PORTBbits.RB1 = 1;
    TRISBbits.TRISB1 = 0;
    TRISBbits.TRISB0 = LATBbits.LATB0 = 0;
    
    //let things settle down.
    //Delay10KTCYx(100);
    

//	          LATBbits.LATB3 = LATBbits.LATB2 = 0;
//          TRISBbits.TRISB3 = TRISBbits.TRISB2 = 0;

/*    InitTempSensor();
    mInitPOT();
    ADCON2bits.ADFM = 1;   // ADC result right justified
  
    ResetTempLog();
    temp_mode = TEMP_REAL_TIME;
    */
    /* Init Timer0 for data logging interval (every 1 second) */
    //T0CON = 0b10010111;
    //T0CONbits.T08BIT = 0;       // 16-bit mode
    //T0CONbits.T0CS = 0;         // Select Fosc/4
    //T0CONbits.PSA = 0;          // Assign prescalar (default is /256)
    /* Timer0 is already enabled by default */
}//end UserInit


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    BlinkUSBStatus();
    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
    
    ServiceRequests();

    if(temp_mode == TEMP_LOGGING)
    {
        if(INTCONbits.TMR0IF == 1)
        {
            INTCONbits.TMR0IF = 0;          // Clear flag
            TMR0H = TIMER0H_VAL;
            TMR0L = TIMER0L_VAL;            // Reinit timer value;

            if(AcquireTemperature())
            {
                temp_data[pTemp] = temperature._word;
                
                // First update valid_temp
                if(valid_temp < 30)         // 30 data points max
                    valid_temp++;
                    
                // Next update pTemp
                if(pTemp == 29)
                    pTemp = 0;
                else
                    pTemp++;
            }//end if
        }//end if
    }//end if
}//end ProcessIO

void ResetTempLog(void)
{
    pTemp = 0;
    valid_temp = 0;
}//end ResetLog

void ReadPOT(void)
{
    ADCON0bits.GO = 1;              // Start AD conversion
    while(ADCON0bits.NOT_DONE);     // Wait for conversion
    return;
}//end ReadPOT
#define PGC PORTBbits.RB2
#define PGD PORTBbits.RB3
#define PGDTRIS TRISBbits.TRISB3

void send4bitcmd(char piccmd) {
  char i;

  //next two lines in case the last op was a read.
  PGC = 1;
  PGDTRIS = 0; //PGD is now back as an output

  
  for (i=0;i<4;i++) {
    PGD = piccmd & 1;
    PGC = 1;
    PGC = 0;
    piccmd = piccmd >> 1;
  }
}

void send4bitcmdstretch(char piccmd) {
  char i;

  //next two lines in case the last op was a read.
  PGC = 1;
  PGDTRIS = 0; //PGD is now back as an output

  
  for (i=0;i<4;i++) {
    PGD = piccmd & 1;
    PGC = 1;
    if (i==3) Delay10KTCYx(13);
    PGC = 0;
    piccmd = piccmd >> 1;
  }
}

void send8bitdata(byte data) {
  char i;
  for (i=0;i<8;i++) {
    PGD = data & 1;
    PGC = 1;
    
    PGC = 0;
    data = data >> 1;
  }
}

byte receive8bitdata() {
  char i;
  byte data = 0;
  PGDTRIS = 1; //PGD is now an input
  for (i=0;i<8;i++) {
    
    PGC = 1;
    PGC = 0;
    data = data >> 1;
    //data = data | (PGD << 7);
    if (PGD) {
      data = data | 0x80;
    } 
  }

  return data;
}
void doublestretchnop() {
  send4bitcmdstretch(0);
  Delay10KTCYx(13);
  send8bitdata(0);
  send8bitdata(0);
}
void performcmds() {
  int commandptr = 2;
  
  unsigned char cmdcount = dataPacket._byte[1];
  unsigned char i,j;
  counter = 0x01;
  for (i = 0; i<cmdcount;i++) {
    char cmdclass = dataPacket._byte[commandptr] >> 4;
    char piccmd = dataPacket._byte[commandptr] & 0x0F;
    commandptr++;
    switch (cmdclass) {
      case 0: //generic write op
        send4bitcmd(piccmd);
        send8bitdata(dataPacket._byte[commandptr]);        
        commandptr++;
        send8bitdata(dataPacket._byte[commandptr]);        
        commandptr++;

        break;
      case 1: //generic read op
        send4bitcmd(piccmd);
        send8bitdata(0/*dataPacket._byte[commandptr]*/);        
        //commandptr++;
        dataPacket._byte[counter] = receive8bitdata();
        counter ++;
        break;
	  case 2: //write op with delay of just over 10ms between instruction and data
        send4bitcmd(piccmd);
        Delay10KTCYx(13);
        send8bitdata(dataPacket._byte[commandptr]);        
        commandptr++;
        send8bitdata(dataPacket._byte[commandptr]);        
        commandptr++;

        break;
      case 3: //table pointer set sequence
        
        for (j=0xF8;j>=0xf6;j--) {
          send4bitcmd(0);
          send8bitdata(dataPacket._byte[commandptr]);        
          commandptr++;
          send8bitdata(0x0E);
          send4bitcmd(0);
          send8bitdata(j);        
          send8bitdata(0x6E);
        }
        break;
      case 4:
        //nop stretched during last high of command nibble and with delay between command nibble and data
        doublestretchnop();

        break;
      case 5:
      case 7: {
        //5=multiple table write sequence ending in program
        //7=multiple table write sequence not ending in program (for pics with write block size >= 64K)
        int panels = 1 << (piccmd >> 2);
        int writesizeperpanel = 4 << (piccmd & 3);
        byte baseu,baseh,basel;
        int j;
        baseu = dataPacket._byte[commandptr];        
        commandptr++;
        baseh = dataPacket._byte[commandptr];        
        commandptr++;
        basel = dataPacket._byte[commandptr];        
        commandptr++;
        
        for (j = 0;j<panels;j++) {
          int k;
          for (k=0xF8;k>=0xf6;k--) {
            int a;
            send4bitcmd(0);
            switch (k) {
              case 0xF8:
                a = baseu;
                break;
              case 0xF7:
                a = baseh;
                break;
              case 0xF6:
                a = basel;
                break;
            }
            send8bitdata(a);        
            
            send8bitdata(0x0E);
            send4bitcmd(0);
            send8bitdata(k);        
            send8bitdata(0x6E);
          }
        
          for (k =0;k<writesizeperpanel;k++) {
            int cmd;
            if (k==writesizeperpanel-1) {
              if ((j==panels-1) && (cmdclass==5)) {
                cmd = 0b1111;//table write and start programming;
              } else {
                cmd = 0b1100; //table write 
              }
            } else {
              cmd = 0b1101; //table write post increment by 2
            }
            send4bitcmd(cmd);
            send8bitdata(dataPacket._byte[commandptr]);        
            commandptr++;
            send8bitdata(dataPacket._byte[commandptr]);        
            commandptr++;
          }
          baseh += 32 ; 
        }

        break;
      }  
      case 6 : {
        //configuration bits programming main block.
        int j;
        for (j=0;j<=0x0d;j++) {
          send4bitcmd(0xF);
          send8bitdata(dataPacket._byte[commandptr]);        
          send8bitdata(dataPacket._byte[commandptr]);        
          commandptr++;
          doublestretchnop();
          //increment table pointer
          send4bitcmd(0);
          send8bitdata(0xF6);
          send8bitdata(0x2A);
        }
        break;
      }
      
    }
    //Delay1KTCYx(1);
  }
  
}

void ServiceRequests(void)
{
  byte index;
   
  if(USBGenRead((byte*)&dataPacket,sizeof(dataPacket)))
  {
    counter = 0;
    switch(dataPacket.CMD) {
      case READ_VERSION:

        
        //last breaking change was in 0.00.01 //not really, thats just when i got round to adding a version command ;)
        dataPacket._byte[0] = 0x00;
        dataPacket._byte[1] = 0x00;
        dataPacket._byte[2] = 0x01;

        //current version is 0.00.01
        dataPacket._byte[3] = 0x00;
        dataPacket._byte[4] = 0x00;
        dataPacket._byte[5] = 0x02;
        counter = 0x08;
		break;

      case SET_VPP:
        
        if (dataPacket._byte[1] == 0) { //if bringing up vpp
          //setup pgc and pgd outputs.
          LATBbits.LATB3 = LATBbits.LATB2 = 0;
          TRISBbits.TRISB3 = TRISBbits.TRISB2 = 0;
    	} 
        PORTBbits.RB1 = dataPacket._byte[1];
        if (dataPacket._byte[1] == 1) { //if taking down vpp
          //make pgc and pgd hi-z
          LATBbits.LATB3 = LATBbits.LATB2 = 1;    	
        }
        counter = 0x01;
      break;
      case TEST_COMS:
        dataPacket._byte[0] = 'D';
        dataPacket._byte[1] = 'E';
        dataPacket._byte[2] = 'A';
        dataPacket._byte[3] = 'D';
        dataPacket._byte[4] = 'B';
        dataPacket._byte[5] = 'E';
        dataPacket._byte[6] = 'E';
        dataPacket._byte[7] = 'F';
        counter = 0x08;
        break;
      case PERFORM_COMMANDS: 
        performcmds();
        //counter = 0x02;
        break;
      default:
        break;
    }//end switch()
    if(counter != 0) {
      if(!mUSBGenTxIsBusy())
        USBGenWrite((byte*)&dataPacket,counter);
    }//end if
  }//end if

}//end ServiceRequests

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;
}
    //#define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    //#define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    //#define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    //#define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}


void TXbyte(byte data)
{
    while(TXSTAbits.TRMT==0);
    TXREG = data;
}//end TXbyte


/** EOF user.c ***************************************************************/
