program picsquirt;

uses
  usbprocs,
  picprocs,
  inifiles,
  sysutils,
  {$ifdef win32}
    //needed for dword in delphi 3, otherwise harmless on windows
    windows,
  {$endif}
  hexfileread in 'hexfileread.pas',
  globaldata in 'globaldata.pas';

const
{$ifdef win32}
  {$R *.RES}
  dirseperator = '\';
{$else}
  dirseperator = '/';
{$endif}



const
  requiredversion : array[0..2] of byte = ($00,$00,$01);

  //for pics with big write blocks
  requiredversionbig : array[0..2] of byte = ($00,$00,$02);
function versionstr(var version) : string;
var
  versiona : array[0..2] of byte absolute version;
begin
  result := inttohex(versiona[0],2)+'.'+inttohex(versiona[1],2)+'.'+inttohex(versiona[2],2);
end;
function compareversions(var version1,version2) : integer;
//<0 1<2
//0 1=2
//>0 1>2
var
  version1a : array[0..2] of byte absolute version1;
  version2a : array[0..2] of byte absolute version2;
  cmp1 : integer;
  cmp2 : integer;
begin
  if version1a[0] <> version2a[0] then begin
    cmp1 := version1a[0];
    cmp1 := version2a[0];
  end else begin
    if version1a[1] <> version2a[1] then begin
      cmp1 := version1a[1];
      cmp2 := version2a[1];
    end else begin
      cmp1 := version1a[2];
      cmp2 := version2a[2];
    end;
  end;
  result := cmp1-cmp2;
end;

var
  firmwareversion : array[0..2] of byte;

procedure checkversion;
var
    send_buf,receive_buf : array[0..63] of byte;
    RecvLength : dword;

begin
  send_buf[0] := READ_VERSION;      // Command
    //send_buf[1] := value;

    RecvLength:=8; // expected receive length

    if(SendReceivePacket(@send_buf[0],2,@receive_buf[0],@RecvLength,1000,1000) = 1) then begin
       receive_buf[8] := 0;
       writeln('firmware version:'+versionstr(receive_buf[3])+' last break:'+versionstr(receive_buf[0])+' required'+versionstr(requiredversion));
       move(receive_buf[3],firmwareversion,3);
       if compareversions(requiredversion,receive_buf[3]) > 0 then raise exception.create('version too low');
       if compareversions(requiredversion,receive_buf[0]) < 0 then raise exception.create('version too high');

    end else
       raise exception.create('USB Operation Failed\r\n');
end;

var
  i,j,k : integer;
  s : string;
  devid : word;
  revision : byte;
  address : integer;
  flag : boolean;
  picspecsfile : string;
  picspecsfileif : tinifile;
begin

  writeln('initialising usb');
  initusb;
  try
    writeln('reading firmware revision');
    checkversion;


    writeln('bringing up vpp');
    setvpp(0);
    writeln('reading device id');
    s := readblock($3FFFFE,2);
    if length(s) = 0 then begin
      raise exception.create('USB Operation Failed\r\n');
      //writeln(recvlength);
    end;
    devid := ord(s[1])+(ord(s[2])*256);
    revision := devid and $001F;
    devid := devid and $FFE0;

    writeln('device found: id=0x'+inttohex(devid,4)+' revision=0x'+inttohex(revision,2));

    picspecsfile := ExtractFileDir(paramstr(0));
    if picspecsfile[length(picspecsfile)] <> dirseperator then picspecsfile := picspecsfile + dirseperator;
    picspecsfile := picspecsfile + 'picspecs.ini';
    picspecsfileif := TIniFile.Create(picspecsfile);
    try
      s := picspecsfileif.ReadString(inttohex(devid,4),'name','');
      if s = '' then raise exception.create('unrecognised device');
      writeln('target device detected is a '+s);
      picspecs.panelsize := picspecsfileif.ReadInteger(inttohex(devid,4),'panelsize',0);
      picspecs.panelcount := picspecsfileif.ReadInteger(inttohex(devid,4),'panelcount',0);
      picspecs.bytestowriteperpanel := picspecsfileif.ReadInteger(inttohex(devid,4),'bytestowriteperpanel',0);
      picspecs.newstyle := picspecsfileif.ReadBool(inttohex(devid,4),'newstyle',false);

      if (picspecs.bytestowriteperpanel > 32) and (compareversions(requiredversionbig,firmwareversion) > 0) then begin
        raise exception.create('pics with a write size over 32 bytes need at least firmware version 0.00.02');
      end;
    finally
      picspecsfileif.free;
    end;



    programmemorysize := picspecs.panelsize*picspecs.panelcount;
    programmemory := allocmem(programmemorysize);
    fillchar(programmemory[0],programmemorysize,#$ff);
    readhexfile(paramstr(1));
    //writeln(integer(programmemory));

    bulkerase;
    //write program memory;
    for i := 0 to (picspecs.panelsize div picspecs.bytestowriteperpanel)-1 do begin
      address := i*picspecs.bytestowriteperpanel;
      flag := false;
      for j := 0 to picspecs.panelcount-1 do begin
        for k := 0 to picspecs.bytestowriteperpanel-1 do begin
          if programmemory[address+(picspecs.panelsize*j)+k]<>$FF then flag := true;
        end;
      end;
      if flag then doprog(address,picspecs.panelcount,picspecs.bytestowriteperpanel);
      if (i and $F) = $F then write('*');
    end;

    writeln;
    writeconfigbits;
  finally
    setvpp(1);
    shutdownusb;
  end;
end.
